/*
 * header file for a program
 * to calculate simplicial collapsing
 *
 * by M.Hachimori, May 2000
 * /

#include <stdio.h>
#include <stdlib.h>

/*#########################################################################*/
/*
 * Some maximum numbers defined here.
 * Should be changed if larger data will be treated
 * in the computation.
 */
#define MAXDIM 10      /* max. of dimension of simplicial complexes */
#define MAXFACE 100000  /* max. of number of faces */
#define MAXLINE 100 /* max. of characters in one line in the input file  */

/*#########################################################################*/

/* simplicial complex */
/* A face is represented by [dim, v_1, v_2, ..., v_(dim+1), X, X, ...],
   a vector with dim+2 entries.
 */  

struct simplices { short vert[MAXDIM + 2];
                   struct simplices *prev;
                   struct simplices *next; };
typedef struct simplices Simplices;      /* a set of simplices */


struct faces { int dim;
                 int n;
                 Simplices *faces; };
typedef struct faces Faces;              /* a set of simplices
                                            with additional information */

typedef struct list_simplices List;     /* a list of "set of simplices" */
struct list_simplices {Simplices *simplices;
                       List *next;};



/**********************************************
       prototypes of functions
 **********************************************/
/* in "io.c" */
int char_to_array(char *ch, short *ans);
void array_to_char(short *arr, char s[]);
Simplices *input_faces(char filename[]);
int cmp(short *a, short *b);
void print_faces(Simplices *simplices);

/* in "tools.c" */
void sort_faces(Simplices *simplices);
void msort(Simplices *simplices,
               int n);
int cmp_faces(short simplices1[], 
              short simplices2[]);
void delete_faces(Simplices *simplices);
int search(short simplex[], Simplices *simplices);
int binarysearch(short simplex[], Simplices *simplices, int n);
int face_contained(short simplex1[], short simplex2[]);
Simplices *copy_faces(Simplices *faces);

/* in "elementary.c" */
Simplices *facets2faces(Simplices *facets);
Simplices *faces2facets(Simplices *faces);
Simplices *boundary_facets(Simplices *facets);
Simplices *pure_part(Simplices *faces, int k);
int binom(int n, int r);
void binom_expand(int n, int i, int bi[]);
int Mseq(int hvec[], int dim);
Simplices *intersection_faces(Simplices *faces1, Simplices *faces2);
Simplices *union_faces(Simplices *faces1, Simplices *faces2);
Simplices *link_faces(Simplices *faces, short face[]);
Simplices *deletion_faces(Simplices *faces, short face[]);
Simplices *stellar_subdivision(Simplices *faces, short face[]);
Simplices *barycentric_subdivision(Simplices *faces);
Simplices *join_faces(Simplices *faces1, Simplices *faces2);
int purity_check(Simplices *faces);
int pm_check(Simplices *faces);
Simplices *bistellar_exchange(Simplices *faces, short a[MAXDIM+2]);
Simplices *intersection_facets(Simplices *faces1, Simplices *faces2);


