/*
 * basic functions dealing with simplicial complexes.
 * by M.Hachimori, May 2000.
 */

#include <stdlib.h>
#include "s_complex.h"


/*
 * function to sort a set of faces
 * using merge sort
 *   (faces are lexicographically ordered and
 *    high dimensional face is larger.)
 */
void sort_faces(simplices)
  Simplices *simplices;
{

  Simplices* current, first, last;
  int n;


  current = simplices;
  n = 0;
  while(current != NULL){
    current = current->next;
    n ++;}

  msort(simplices, n);

  return;

}

void msort(simplices, n)
  Simplices *simplices;
  int n;
{
  Simplices *tmp, *tmp_first, *tmp_last;
  Simplices *simplices1, *simplices2;
  Simplices *tmp1, *tmp2;
  Simplices *current;
  int i, j, n1, n2;

  if (n <= 1) return;

  /* sort two divided parts disjointly */
  current = simplices;
  simplices1 = (Simplices*)malloc(sizeof(Simplices));
  simplices1->prev = NULL;
  tmp1 = simplices1;
  n1 = n/2; n2 = n - n1;
  for (i = 1; i <= n1  ; i ++){
    for (j = 0; j <= current->vert[0]+1; j ++)
      tmp1->vert[j] = current->vert[j];
    current = current->next;
    tmp1->next = (Simplices*)malloc(sizeof(Simplices));
    tmp1->next->prev = tmp1;
    tmp1 = tmp1->next;}
  tmp1 = tmp1->prev;
  free(tmp1->next);
  tmp1->next=NULL;
  simplices2 = (Simplices*)malloc(sizeof(Simplices));
  simplices2->prev = NULL;
  tmp2 = simplices2;
  for (i = n1+1; i <= n  ; i ++){
    if (current == NULL) break;
    for (j = 0; j <= current->vert[0]+1; j ++)
      tmp2->vert[j] = current->vert[j];
    current = current->next;
    tmp2->next = (Simplices*)malloc(sizeof(Simplices));
    tmp2->next->prev = tmp2;
    tmp2 = tmp2->next;}
  tmp2 = tmp2->prev;
  free(tmp2->next);
  tmp2->next=NULL;
  msort(simplices1, n1);
  msort(simplices2, n2);

  /* merge the two parts */
  current = simplices;
  tmp1 = simplices1;
  tmp2 = simplices2;
  while(n1 > 0 || n2 > 0){
    if (n2 == 0 || (n1 > 0 && cmp_faces(tmp1->vert, tmp2->vert) == 1)){
      for(i = 0; i <= tmp1->vert[0] + 1; i ++)
        current->vert[i] = tmp1->vert[i];
      current = current->next;
      tmp1 = tmp1->next;
      n1 --;}
    else{
      for(i = 0; i <= tmp2->vert[0] + 1; i ++)
        current->vert[i] = tmp2->vert[i];
      current = current->next;
      tmp2 = tmp2->next;
      n2 --;}}

   delete_faces(simplices1);
   delete_faces(simplices2);


  return;
}


/* compare function --- return 1 if the first one is smaller. */
int cmp_faces(simplices1, simplices2)
  short simplices1[];
  short simplices2[];
{
   int i;

   /* first compare the dimension (larger is first) */
   if (simplices1[0] > simplices2[0]) return 1;
   if (simplices2[0] > simplices1[0]) return -1;

   /* next compare lexicographically */
   for (i = 1; i <= simplices1[0]+1; i ++){
     if (simplices1[i] > simplices2[i]) return -1;
     if (simplices2[i] > simplices1[i]) return 1;}

   return 0;
}

/* function to delete facelist */
void delete_faces(simplices)
  Simplices *simplices;
{
  Simplices *tmp;

  while(simplices->prev != NULL)
    simplices = simplices->prev;
  while(simplices != NULL){
    tmp = simplices->next;
    free(simplices);
    simplices = tmp;}

  return;
}


/* function to determine whether a simplex is in a set
   of simpices using binary search. */
int search(simplex, simplices)
  short simplex[];
  Simplices *simplices;
{
  Simplices* current;
  int n;


  current = simplices;
  n = 0;
  while(current != NULL){
    current = current->next;
    n ++;}

  return(binarysearch(simplex, simplices, n));

}


/* binary search */
int binarysearch(simplex, simplices, n)
  short simplex[];
  Simplices *simplices;
  int n;
{
  int i, n1, n2;
  Simplices *current, *simplices1, *simplices2;

  if (n < 1) return 0;
  if (n == 1)
    if (cmp_faces(simplex, simplices->vert) == 0)
      return 1;
    else
      return 0;

  /* search two divided parts disjointly */
  current = simplices;
  simplices1 = current;
  n1 = n/2; n2 = n - n1;
  for (i = 1; i <= n1  ; i ++)
    current = current->next;
  simplices2 = current;
  if (binarysearch(simplex, simplices1, n1) == 1 ||
      binarysearch(simplex, simplices2, n2) == 1)
    return 1;
  else
    return 0;

}


/* Function to determine whether simplex1 is a face of simplex2
     return 1 if simplex1 is a face of simplex2, else return 0 */
   /* (Remark: return 1 if simplex1 = simplex2.) */
int face_contained(simplex1, simplex2)
  short simplex1[];
  short simplex2[];
{
  int idx1, idx2;

  if (simplex1[0] > simplex2[0])
    return 0;

  idx1 = 1; idx2 = 1;
  while(1){
    if (simplex1[idx1] == simplex2[idx2]){
      idx1 ++; idx2 ++;
      if (idx1 > simplex1[0]+1)
        return 1;
      else if (idx2 > simplex2[0]+1)
        return 0;
      else
        continue;
      }
    if (simplex1[idx1] < simplex2[idx2])
      return 0;
    idx2 ++;
    if (idx2 > simplex2[0]+1)
      return 0;
    }
}


/*
 * Function to make a copy of  a list of faces
 */
Simplices *copy_faces(faces)
  Simplices *faces;
{
  Simplices *output, *return_address;
  int i;

  if (faces == NULL) return NULL;

  output = (Simplices*)malloc(sizeof(Simplices));
  output->prev = NULL;
  output->next = NULL;
  return_address = output;
  while(faces != NULL){
    output->next = (Simplices*)malloc(sizeof(Simplices));
    output->next->prev = output;
    output = output->next;
    output->next = NULL;
    for (i = 0; i <= faces->vert[0]+1; i ++)
      output->vert[i] = faces->vert[i];
    faces = faces->next;}

  return_address = return_address->next;
  free(return_address->prev);
  return_address->prev = NULL;
  return return_address;
}

/* Function to determine whether simplex1 and simplex2 are identical.
     return 1 if identical, else return 0 */
int face_identical(simplex1, simplex2)
  short simplex1[];
  short simplex2[];
{
  int i;

  if (simplex1[0] != simplex2[0]) return 0;

  for (i = 0; i <= simplex1[0] + 1; i ++)
    if (simplex1[i] != simplex2[i])
      return 0;
  return 1;
}

